@@ -43,8 +43,6 @@
 
 #include "mcc.h"
 
-#define uninitialized_var(x) x = x
-
 #define OFONO_SERVICE			"org.ofono"
 
 #define OFONO_MANAGER_INTERFACE		OFONO_SERVICE ".Manager"
@@ -834,7 +832,7 @@ static void extract_ipv6_settings(DBusMessageIter *array,
 {
 	DBusMessageIter dict;
 	char *address = NULL, *gateway = NULL;
-	unsigned char prefix_length;
+	unsigned char prefix_length = 0;
 	char *nameservers = NULL;
 	const char *interface = NULL;
 	int index = -1;
@@ -937,7 +935,7 @@ static connman_bool_t ready_to_create_device(struct modem_data *modem)
 static void create_device(struct modem_data *modem)
 {
 	struct connman_device *device;
-	char *uninitialized_var(ident);
+	char *ident = NULL;
 
 	DBG("%s", modem->path);
 
@@ -1062,7 +1060,7 @@ static void remove_network(struct modem_data *modem)
 static int add_cm_context(struct modem_data *modem, const char *context_path,
 				DBusMessageIter *dict)
 {
-	const char *context_type;
+	const char *context_type = NULL;
 	struct network_context *context = NULL;
 	connman_bool_t active = FALSE;
 
